@@ -1,4 +1,4 @@
-recursive_search <- function(tc, qlist, subcontext=NULL, feature='token', mode = c('unique_hits','features','contexts'), parent_relation='', all_case_sensitive=FALSE, all_ghost=FALSE, all_flag_query=list(), level=1) {
+recursive_search <- function(tc, qlist, subcontext=NULL, feature='token', mode = c('unique_hits','features','contexts'), parent_relation='', all_case_sensitive=FALSE, all_ghost=FALSE, all_flag_query=list(), keep_longest=TRUE, as_ascii=F, level=1) {
   .ghost = NULL; .term_i = NULL; .seq_i = NULL; .group_i = NULL ## for solving CMD check notes (data.table syntax causes "no visible binding" message)
 
   mode = match.arg(mode) ## 'unique_hit' created complete and unique sets of hits (needed for counting) but doesn't assign all features
@@ -12,13 +12,14 @@ recursive_search <- function(tc, qlist, subcontext=NULL, feature='token', mode =
   if (!qlist$feature == "") feature = qlist[['feature']]  ## if the query specifies a feature column, override the feature parameter
   for (n in names(qlist$all_flag_query)) all_flag_query[[n]] = unique(c(all_flag_query[[n]], qlist$all_flag_query[[n]]))
 
+  qlist = collapse_or_queries(qlist)
 
   nterms = length(qlist$terms)
   for (j in 1:nterms) {
     q = qlist$terms[[j]]
     is_nested = 'terms' %in% names(q)
     if (is_nested) {
-      jhits = recursive_search(tc, q, subcontext=subcontext, feature='token', mode=mode, parent_relation=qlist$relation, all_case_sensitive, all_ghost, level=level+1)
+      jhits = recursive_search(tc, q, subcontext=subcontext, feature='token', mode=mode, parent_relation=qlist$relation, all_case_sensitive, all_ghost, all_flag_query, keep_longest, as_ascii, level=level+1)
       if (nterms == 1) {
         if (level == 1 & mode == 'contexts' & !is.null(jhits)) jhits = unique(subset(jhits, select=c('doc_id',subcontext)))
         return(jhits)
@@ -37,7 +38,7 @@ recursive_search <- function(tc, qlist, subcontext=NULL, feature='token', mode =
       for (n in names(all_flag_query)) flag_query[[n]] = unique(c(flag_query[[n]], all_flag_query[[n]]))
 
       only_context = mode == 'contexts' & qlist$relation %in% c('AND','NOT')  ## only for AND and NOT, because proximity and sequence require feature positions (and OR can be nested in them)
-      jhits = tc$lookup(q$term, feature=feature, ignore_case=!.case_sensitive, sub_query=flag_query, only_context=only_context, subcontext=subcontext)
+      jhits = tc$lookup(q$term, feature=feature, ignore_case=!.case_sensitive, sub_query=flag_query, only_context=only_context, subcontext=subcontext, as_ascii=as_ascii)
       if (!is.null(jhits)) {
         jhits[, .ghost := .ghost]
         if (qlist$relation %in% c('proximity','sequence','AND')) jhits[,.group_i := paste0(j, '_')] else jhits[,.group_i := ''] ## for keeping track of nested multi word queries
@@ -53,6 +54,7 @@ recursive_search <- function(tc, qlist, subcontext=NULL, feature='token', mode =
   }
 
   hits = data.table::rbindlist(hit_list, fill=TRUE)
+
   if (nrow(hits) == 0) return(NULL)
 
   feature_mode = mode == 'features'
@@ -70,10 +72,31 @@ recursive_search <- function(tc, qlist, subcontext=NULL, feature='token', mode =
     hits = subset(hits, hit_id > 0)
   }
   if (nrow(hits) == 0) return(NULL)
+
+  if (level == 1 & mode == 'unique_hits') hits = remove_duplicate_hit_id(hits, keep_longest)
   if (level == 1 & mode == 'contexts') hits = unique(subset(hits, select=c('doc_id',subcontext)))
+
   return(hits)
 }
 
+collapse_or_queries <- function(qlist) {
+  if (qlist$relation == 'OR') {
+    nested = sapply(qlist$terms, function(x) 'terms' %in% names(x))
+    has_flag_query = sapply(qlist$terms, function(x) length(x$flag_query) > 0)
+    select = !nested & !has_flag_query # these terms are collapse-able
+
+    if (sum(select) > 1) {
+      terms = sapply(qlist$terms[select], function(x) x[c('case_sensitive','ghost','term')], simplify = F)
+      terms = data.table::rbindlist(terms)
+
+      col_terms = stats::aggregate(term ~ case_sensitive + ghost, data=terms, FUN = c, simplify=F)
+      col_terms = apply(col_terms, 1, as.list)
+      col_terms = sapply(col_terms, function(x) c(x, list(flag_query=list())), simplify = F)
+      qlist$terms = c(col_terms, qlist$terms[!select])
+    }
+  }
+  qlist
+}
 
 
 get_query_code <- function(query, code=NULL) {
@@ -82,75 +105,93 @@ get_query_code <- function(query, code=NULL) {
   hashcode = ifelse(hashcount == 1, stringi::stri_replace(query, '$1', regex = '([^\\\\])#.*'), NA)
 
   if (!is.null(code)) {
+    code = as.character(code)
     if (!length(code) == length(query)) stop('code and query vectors need to have the same length')
     code = ifelse(is.na(code), hashcode, code)
   } else code = hashcode
+  #print(code)
   code[is.na(code)] = paste('query', 1:sum(is.na(code)), sep='_')
   if (anyDuplicated(code)) stop('Cannot have duplicate codes')
   code
 }
 
-remove_query_label <- function(query) stringi::stri_replace(query, '', regex = '.*([^\\\\])# ?')
+remove_query_label <- function(query) {
+  ht_count = stringi::stri_count(query, fixed='#')
+  ht_nolabel_count = stringi::stri_count(query, regex='\\\\#')
+  has_label = (ht_count - ht_nolabel_count) > 0
+
+  if (any(has_label)) {
+    query[has_label] = unlist(sapply(stringi::stri_split_fixed(query[has_label], pattern = '#', n=2), function(x) x[[2]]))
+  }
+  query
+}
 
 get_sequence_hit <- function(d, seq_length, subcontext=NULL){
   hit_id = NULL ## used in data.table syntax, but need to have bindings for R CMD check
-  setorderv(d, c('doc_id', 'token_i', '.term_i'))
+  setorderv(d, c('doc_id', 'token_id', '.term_i'))
   if (!is.null(subcontext)) subcontext = d[[subcontext]]
-  .hit_id = .Call('_corpustools_sequence_hit_ids', PACKAGE = 'corpustools', as.integer(d[['doc_id']]), as.integer(subcontext), as.integer(d[['token_i']]), as.integer(d[['.term_i']]), seq_length)
+  .hit_id = .Call('_corpustools_sequence_hit_ids', PACKAGE = 'corpustools', as.numeric(d[['doc_id']]), as.numeric(subcontext), as.numeric(d[['token_id']]), as.numeric(d[['.term_i']]), seq_length)
   d[,hit_id := .hit_id]
 }
 
 get_proximity_hit <- function(d, n_unique, window=NA, subcontext=NULL, seq_i=NULL, replace=NULL, feature_mode=F, directed=F){
   hit_id = NULL ## used in data.table syntax, but need to have bindings for R CMD check
-  setorderv(d, c('doc_id', 'token_i', '.term_i'))
+  setorderv(d, c('doc_id', 'token_id', '.term_i'))
   if (!is.null(subcontext)) subcontext = d[[subcontext]]
   if (!is.null(seq_i)) seq_i = d[[seq_i]]
   if (!is.null(replace)) replace = d[[replace]]
 
-  .hit_id = .Call('_corpustools_proximity_hit_ids', PACKAGE = 'corpustools', as.integer(d[['doc_id']]), as.integer(subcontext), as.integer(d[['token_i']]), as.integer(d[['.term_i']]), n_unique, window, as.numeric(seq_i), replace, feature_mode, directed)
+  .hit_id = .Call('_corpustools_proximity_hit_ids', PACKAGE = 'corpustools', as.numeric(d[['doc_id']]), as.numeric(subcontext), as.numeric(d[['token_id']]), as.numeric(d[['.term_i']]), n_unique, window, as.numeric(seq_i), replace, feature_mode, directed)
   d[,hit_id := .hit_id]
 }
 
 get_AND_hit <- function(d, n_unique, subcontext=NULL, group_i=NULL, replace=NULL, feature_mode=F){
   hit_id = NULL ## used in data.table syntax, but need to have bindings for R CMD check
-  setorderv(d, c('doc_id', 'token_i', '.term_i'))
+  setorderv(d, c('doc_id', 'token_id', '.term_i'))
   if (!is.null(subcontext)) subcontext = d[[subcontext]]
   if (!is.null(group_i)) group_i = d[[group_i]]
   if (!is.null(replace)) replace = d[[replace]]
-
-  .hit_id = .Call('_corpustools_AND_hit_ids', PACKAGE = 'corpustools', as.integer(d[['doc_id']]), as.integer(subcontext), as.integer(d[['token_i']]), as.integer(d[['.term_i']]), n_unique, as.character(group_i), replace, feature_mode)
+  .hit_id = .Call('_corpustools_AND_hit_ids', PACKAGE = 'corpustools', as.numeric(d[['doc_id']]), as.numeric(subcontext), as.numeric(d[['token_id']]), as.numeric(d[['.term_i']]), n_unique, as.character(group_i), replace, feature_mode)
   d[,hit_id := .hit_id]
 }
 
 get_OR_hit <- function(d) {
   hit_id = NULL ## used in data.table syntax, but need to have bindings for R CMD check
+
   if (!'hit_id' %in% colnames(d)) {
-    i = 1:nrow(d)
-  } else i = d$hit_id
-  isna = is.na(i)
+    .hit_id = 1:nrow(d)
+  } else .hit_id = d$hit_id
+
+  isna = is.na(.hit_id)
   if (any(isna)) {
-    if (all(isna)) na_ids = 1:length(i) else na_ids = 1:sum(isna) + max(i, na.rm = T)
-    i[isna] = na_ids
+    if (all(isna)) na_ids = 1:length(.hit_id) else na_ids = 1:sum(isna) + max(.hit_id, na.rm = T)
+    .hit_id[isna] = na_ids
   }
-  .hit_id = i
+
+  if ('.term_i' %in% colnames(d)) .hit_id = global_id(d$.term_i, .hit_id)
   d[,hit_id := .hit_id]
 }
 
-grep_global_i <- function(fi, regex, ...) {
-  exact_feature = levels(fi$feature)[grepl(regex, levels(fi$feature), ...)]
-  fi[list(exact_feature),,nomatch=0]$global_i
+remove_duplicate_hit_id <- function(d, keep_longest=TRUE) {
+  .hit_id_length = NULL; .ghost = NULL; hit_id = NULL ## for solving CMD check notes (data.table syntax causes "no visible binding" message)
+  if (!'token_id' %in% colnames(d)) return(d)
+
+  dup = duplicated(d[,c('doc_id','token_id')])
+  if (any(dup)) {
+    if (!'.ghost' %in% colnames(d)) d$.ghost = F
+
+    if (keep_longest) {
+      d[, .hit_id_length := sum(!.ghost), by=hit_id]   ## count non ghost terms per hit_id
+      pd = d[order(-d$.hit_id_length),]                ## sort by this count to keep duplicates with highest score
+      dup_id = pd$hit_id[duplicated(pd[,c('doc_id','token_id')]) & !d$.ghost]
+      d[, .hit_id_length := NULL]
+    } else {
+      dup_id = d$hit_id[dup & !d$.ghost]
+    }
+    d = subset(d, !hit_id %in% unique(dup_id))
+  }
+  d
 }
 
-grep_fi <- function(fi, regex, ...) {
-  exact_feature = levels(fi$feature)[grepl(regex, levels(fi$feature), ...)]
-  i = fi[list(exact_feature),,nomatch=0]
 
-}
 
-expand_window <- function(i, window) {
-  unique(rep(i, window*2 + 1) + rep(-window:window, each=length(i)))
-}
-
-overlapping_windows <- function(hit_list, window=window) {
-  Reduce(intersect, sapply(hit_list, expand_window, window=window, simplify = F))
-}
