@@ -1,9 +1,7 @@
 #' Find tokens using a Lucene-like search query
 #'
 #' @description
-#' Search tokens in a tokenlist using a query that consists of an keyword, and optionally a condition. For a detailed explanation of the query language please consult the query_tutorial markdown file. For a quick summary see the details below.
-#'
-#' Note that the query arguments (keyword, condition, code, condition_once) can be vectors to search multiple queries at once. Alternatively, the queries argument can be used to pass these arguments in a data.frame
+#' Search tokens in a tokenlist using Lucene-like queries. For a detailed explanation of the query language, see the details below.
 #'
 #' @section Usage:
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
@@ -18,6 +16,8 @@
 #' @param feature The name of the feature column within which to search.
 #' @param mode There are two modes: "unique_hits" and "features". The "unique_hits" mode prioritizes finding full and unique matches., which is recommended for counting how often a query occurs. However, this also means that some tokens for which the query is satisfied might not assigned a hit_id. The "features" mode, instead, prioritizes finding all tokens, which is recommended for coding coding features (the code_features and search_recode methods always use features mode).
 #' @param context_level Select whether the queries should occur within while "documents" or specific "sentences".
+#' @param keep_longest If TRUE, then overlapping in case of overlapping queries strings in unique_hits mode, the query with the most separate terms is kept. For example, in the text "mr. Bob Smith", the query [smith OR "bob smith"] would match "Bob" and "Smith". If keep_longest is FALSE, the match that is used is determined by the order in the query itself. The same query would then match only "Smith".
+#' @param as_ascii if TRUE, perform search in ascii.
 #' @param verbose If TRUE, progress messages will be printed
 #'
 #' @details
@@ -126,7 +126,7 @@
 #' \dontrun{
 #' ## advanced queries
 #' tc = tokens_to_tcorpus(corenlp_tokens, doc_col = 'doc_id',
-#'                        sent_i_col = 'sentence', token_i_col = 'id')
+#'                        sentence_col = 'sentence', token_id_col = 'id')
 #' head(tc$get()) ## search in multiple feature columns with "columnname: "
 #'
 #' ## using the sub/flag query to find only mary as a direct object
@@ -150,20 +150,63 @@
 #' hits = tc$search_features('"(relation: nsubj) say*"')
 #' hits$hits
 #' }
-tCorpus$set('public', 'search_features', function(query, code=NULL, feature='token', mode = c('unique_hits','features'), context_level = c('document','sentence'), verbose=F){
-  search_features(self, query, code=code, feature=feature, mode=mode, context_level=context_level, verbose=verbose)
+tCorpus$set('public', 'search_features', function(query, code=NULL, feature='token', mode = c('unique_hits','features'), context_level = c('document','sentence'), keep_longest=T, as_ascii=F, verbose=F){
+  search_features(self, query, code=code, feature=feature, mode=mode, context_level=context_level, keep_longest=keep_longest, as_ascii=as_ascii, verbose=verbose)
 })
 
-tCorpus$set('public', 'code_features', function(query, code=NULL, feature='token', column='code', verbose=F){
-  hits = search_features(self, query, code=code, feature=feature, mode='features', verbose=verbose)
 
-  .i = self$token_i(doc_id = hits$hits$doc_id, token_i = hits$hits$token_i)
-  .value = hits$hits$code
+#' Code features in a tCorpus based on a search string
+#'
+#' @description
+#' Add a column to the token data that contains a code (the query label) for tokens that match the query (see \link{tCorpus$search_features}).
+#'
+#' @section Usage:
+#' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
+#'
+#' \preformatted{
+#' code_features(query, code=NULL, feature='token', column='code', ...)
+#' }
+#'
+#' @param query A character string that is a query. See \link{search_features} for documentation of the query language.
+#' @param code The code given to the tokens that match the query (usefull when looking for multiple queries). Can also put code label in query with # (see details)
+#' @param feature The name of the feature column within which to search.
+#' @param column The name of the column that is added to the data
+#' @param add_column list of name-value pairs, used to add additional columns. The name will become the column name, and the value should be a vector of the same length as the query vector.
+#' @param context_level Select whether the queries should occur within while "documents" or specific "sentences".
+#' @param keep_longest If TRUE, then overlapping in case of overlapping queries strings in unique_hits mode, the query with the most separate terms is kept. For example, in the text "mr. Bob Smith", the query [smith OR "bob smith"] would match "Bob" and "Smith". If keep_longest is FALSE, the match that is used is determined by the order in the query itself. The same query would then match only "Smith".
+#' @param as_ascii if TRUE, perform search in ascii.
+#' @param verbose If TRUE, progress messages will be printed
+#' @param ... alternative way to specify name-value pairs for adding additional columns
+#'
+#' @name tCorpus$code_features
+#' @examples
+#' tc = create_tcorpus('Anna and Bob are secretive')
+#'
+#' tc$code_features(c("actors# anna bob", "associations# secretive"))
+#' tc$get()
+#' @aliases code_features
+tCorpus$set('public', 'code_features', function(query, code=NULL, feature='token', column='code', add_column=list(), context_level = c('document','sentence'), keep_longest=T, as_ascii=F, verbose=F, ...){
+  codelabel = get_query_code(query, code)
+  code = 1:length(query)
+  hits = search_features(self, query, code=code, feature=feature, mode='features', context_level=context_level, keep_longest=keep_longest, as_ascii=as_ascii, verbose=verbose)
+
+  .i = self$token_id(doc_id = hits$hits$doc_id, token_id = hits$hits$token_id)
+  .value = codelabel[as.numeric(hits$hits$code)]
   self$set(column=column, subset=.i, value=.value, subset_value=F)
 
+  add_column = c(add_column, list(...))
+  if (length(add_column) > 0) {
+    for (i in 1:length(add_column)) {
+      .value = add_column[[i]][as.numeric(hits$hits$code)]
+      self$set(column=names(add_column)[i], subset=.i, value=.value, subset_value=F)
+    }
+  }
+
   invisible(self)
 })
 
+
+
 #' Recode features in a tCorpus based on a search string
 #'
 #' @description
@@ -178,19 +221,20 @@ tCorpus$set('public', 'code_features', function(query, code=NULL, feature='token
 #'
 #' @param feature The feature in which to search
 #' @param new_value the character string with which all features that are found are replaced
-#' @param ... See \link{tCorpus$search_features} for the query parameters
+#' @param query See \link{tCorpus$search_features} for the query parameters
+#' @param ... Additional search_features parameters. See \link{tCorpus$search_features}
 #'
 #' @name tCorpus$search_recode
-#' @aliases search_recode.tCorpus
-tCorpus$set('public', 'search_recode', function(feature, new_value, query){
-  hits = search_features(self, query, feature=feature, mode='features')
-  .i = self$token_i(doc_id = hits$hits$doc_id, token_i = hits$hits$token_i)
+#' @aliases search_recode
+tCorpus$set('public', 'search_recode', function(feature, new_value, query, ...){
+  hits = search_features(self, query, feature=feature, mode='features', ...)
+  .i = self$token_id(doc_id = hits$hits$doc_id, token_id = hits$hits$token_id)
   .new_value = new_value
   self$set(feature, .new_value, subset = .i)
   invisible(self)
 })
 
-search_features <- function(tc, query, code=NULL, feature='token', mode = c('unique_hits','features'), context_level=c('document','sentence'), verbose=F){
+search_features <- function(tc, query, code=NULL, feature='token', mode = c('unique_hits','features'), context_level=c('document','sentence'), keep_longest=TRUE, as_ascii=F, verbose=F){
   .ghost = NULL ## for solving CMD check notes (data.table syntax causes "no visible binding" message)
   is_tcorpus(tc, T)
   mode = match.arg(mode)
@@ -199,13 +243,13 @@ search_features <- function(tc, query, code=NULL, feature='token', mode = c('uni
   if (!feature %in% tc$names) stop(sprintf('Feature (%s) is not available. Current options are: %s', feature, paste(tc$feature_names, collapse=', ')))
   codelabel = get_query_code(query, code)
   query = remove_query_label(query)
-
-  subcontext = if(context_level == 'sentence') 'sent_i' else NULL
+  subcontext = if(context_level == 'sentence') 'sentence' else NULL
   hits = vector('list', length(query))
+
   for (i in 1:length(query)) {
-    if (verbose) print(code[i])
+    if (verbose) cat(as.character(codelabel[i]), '\n')
     q = parse_query(as.character(query[i]))
-    h = recursive_search(tc, q, subcontext=subcontext, feature=feature, mode = mode)
+    h = recursive_search(tc, q, subcontext=subcontext, feature=feature, mode = mode, keep_longest=keep_longest, as_ascii=as_ascii)
     if (!is.null(h)) {
       h[, code := codelabel[i]]
       hits[[i]] = h
@@ -213,13 +257,12 @@ search_features <- function(tc, query, code=NULL, feature='token', mode = c('uni
   }
   hits = data.table::rbindlist(hits)
 
-
   if (nrow(hits) > 0) {
     data.table::setnames(hits, feature, 'feature')
-    setorderv(hits, c('doc_id','token_i'))
+    setorderv(hits, c('doc_id','token_id'))
     hits = subset(hits, subset=!.ghost)
   } else {
-    hits = data.frame(code=factor(), feature=factor(), doc_id=factor(), sent_i=numeric(), token_i = numeric(), hit_id=numeric())
+    hits = data.frame(code=factor(), feature=factor(), doc_id=factor(), sentence=numeric(), token_id = numeric(), hit_id=numeric())
   }
   queries = data.frame(code=codelabel, query=query)
   featureHits(hits, queries)
