@@ -10,7 +10,7 @@ test_that("preprocessing works", {
                       token = c('Renewable','fuel','is','better','than','fossil','fuels','!','A','fueled','debate','about','fuel','Mark','Rutte','is','simply','Rutte'))
   meta = data.frame(document = c(1,2,3), medium=c('a','b','c'))
 
-  tc = tokens_to_tcorpus(tokens, doc_col ='document', token_i_col = 'id', meta=meta)
+  tc = tokens_to_tcorpus(tokens, doc_col ='document', token_id_col = 'id', meta=meta)
 
   dtm = tc$dtm('token')
   ## test preprocess_feature
